/*
 * BEGIN_DESC
 * 
 *  File:     if.h                       Date: 11/30/1999
 *  @(#)B11.23_LR
 * 
 *  Purpose:
 *      <<please update with a synopis of the functionality provided by this file>>
 * 
 *  Classification:			Release to Release Consistency Req:
 *		<<please select one of the following:>>
 * 	kernel subsystem private		none
 * 	kernel private				none
 * 	kernel 3rd party private		limited source
 * 	public					binary & source
 * 
 *  BE header:  yes
 *
 *  Shipped:  yes
 *	/usr/include/net/if.h
 *	/usr/conf/net/if.h
 *
 *  <<please delete the following note if this is a "public" header>>
 *  NOTE:
 *	This header file contains information specific to the internals
 *	of the HP-UX implementation.  The contents of this header file
 *	are subject to change without notice.  Such changes may affect
 *	source code, object code, or binary compatibility between
 *	releases of HP-UX.  Code which uses the symbols contained within
 *	this header file is inherently non-portable (even between HP-UX
 *	implementations).
 * 
 * END_DESC  
*/


#ifndef _NET_IF_INCLUDED
#define _NET_IF_INCLUDED
/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	if.h	7.3 (Berkeley) 6/27/88 plus MULTICAST 1.1
 */

#include <sys/socket.h>
#include <net/netmp.h>

/* Structure large enough to hold any socket address (with the historical
   exception of AF_UNIX).  We reserve 128 bytes.  */
#include <sys/types.h>
#if (~0UL) > 0xffffffff
#define __ss_align	uint64_t
#else
#define __ss_align	uint32_t
#endif
#define _SS_SIZE	128
#define _SS_PADSIZE	(_SS_SIZE - (2 * sizeof(__ss_align)))

struct sockaddr_storage 
  {
    __SOCKADDR_COMMON (__ss_);	/* Address family, etc.  */
    __ss_align __ss_align;	/* Force desired alignment.  */
    char __ss_padding[_SS_PADSIZE];
  };


+

/*
 * Structure defining a queue for a network interface.
 *
 * (Would like to call this struct ``if'', but C isn't PL/1.)
 */
struct ifnet {
	char	*if_name;		/* name, e.g. ``en'' or ``lo'' */
	short	if_unit;		/* sub-unit for lower level driver */
	u_short	if_mtu;			/* maximum transmission unit */
	int	if_index;	
	u_int	if_flags;		/* up/down, broadcast, etc. */
	short	if_timer;		/* time 'til if_watchdog called */
	short	if_pad;			/* if_pad - unused */
	int	if_metric;		/* routing metric (external only) */
	struct	ifaddr *if_addrlist;	/* linked list of addresses per if */
	struct	ifqueue {
		struct	mblk *ifq_head; /* Changed to mblk JAGad94239 */
		struct	mblk *ifq_tail; /* Changed to mblk JAGad94239 */
		int	ifq_len;
		int	ifq_maxlen;
		int	ifq_drops;
		lock_t	*ifq_slock;
	} if_snd;			/* output queue */
	lock_t 	*if_slock;
/* procedure handles */
	int	(*if_init)();		/* init routine */
	int	(*if_output)();		/* output routine */
	int	(*if_ioctl)();		/* ioctl routine */
	int	(*if_control)();	/* kernel control routine */
	int	(*if_watchdog)();	/* timer routine */
/* generic interface statistics */
	int	if_ipackets;		/* packets received on interface */
	int	if_ierrors;		/* input errors on interface */
	int	if_opackets;		/* packets sent on interface */
	int	if_oerrors;		/* output errors on interface */
	int	if_collisions;		/* collisions on csma interfaces */
	lock_t	*if_stat_lock;
/* end statistics */
	struct	ifnet *if_next;
};

#define	IFF_UP		0x1		/* interface is up */
#define	IFF_BROADCAST	0x2		/* broadcast address valid */
#define	IFF_DEBUG	0x4		/* turn on debugging */
#define	IFF_LOOPBACK	0x8		/* is a loopback net */
#define	IFF_POINTOPOINT	0x10		/* interface is point-to-point link */
#define	IFF_NOTRAILERS	0x20		/* avoid use of trailers */
#define	IFF_RUNNING	0x40		/* resources allocated */
#define	IFF_NOARP	0x80		/* no address resolution protocol */
/* next two not supported now, but reserved: */
#define	IFF_PROMISC	0x100		/* receive all packets */
#define	IFF_ALLMULTI	0x200		/* receive all multicast packets */
/* for host requirements: */
#define IFF_LOCALSUBNETS 0x400		/* subnets off this net are local */
#define	IFF_MULTICAST	0x800		/* supports multicast */
/*
 * The IFF_MULTICAST flag indicates that the network can support the
 * transmission and reception of higher-level (e.g., IP) multicast packets.
 * It is independent of hardware support for multicasting; for example,
 * point-to-point links or pure broadcast networks may well support
 * higher-level multicasts.
 */
/* next two used for hardware checksum and copy avoidance */
#define	IFF_CKO		0x1000		/* interface supports header cksum */
#define	IFF_NOACC	0x2000		/* no data access on outbound */
/* Required for ATR driver */
#define IFF_OACTIVE	0x4000		/* transmission in progress */

#define IFF_NOSR8025    0x8000          /* no source route 802.5 */
#define IFF_CKO_ETC	0x10000		/* interface supports trailer cksum */

#define IFF_AR_SR8025   0x20000         /* all routes broadcast for arp-8025 */
#define IFF_ALT_SR8025  0x40000         /* alternating no rif, rif for arp on*/
#define	IFF_MULTI_BCAST	0x80000		/* multicast using broadcast address */
#define	IFF_UNNUMBERED	0x100000	/* non-unique local address */
#define IFF_LFP         0x200000        /* for RFC 1323 - Long-fat pipe */
#define IFF_LNP         0x400000        /* for slow, narrow pipl such as PPP */
#define IFF_NOTIFY_IF_CHANGE 0x800000   /* Notify DLPI when the interface goes
                                           up or down or switches over to another */
#define IFF_PRIORITY         0x80000000      /* VLAN IP precendence enablement */


/*
 * The ifaddr structure contains information about one address
 * of an interface.  They are maintained by the different address families,
 * are allocated and attached when an address is set, and are linked
 * together so all addresses for an interface can be located.
 */
struct ifaddr {
	struct	sockaddr ifa_addr;	/* address of interface */
	union {
		struct	sockaddr ifu_broadaddr;
		struct	sockaddr ifu_dstaddr;
	} ifa_ifu;
#define	ifa_broadaddr	ifa_ifu.ifu_broadaddr	/* broadcast address */
#define	ifa_dstaddr	ifa_ifu.ifu_dstaddr	/* other end of p-to-p link */
	struct	ifnet *ifa_ifp;		/* back-pointer to interface */
	struct	ifaddr *ifa_next;	/* next address for interface */
};

/*
 * Interface request structure used for socket
 * ioctl's.  All interface ioctl's must have parameter
 * definitions which begin with ifr_name.  The
 * remainder may be interface specific.
 */
/* Note: The extended flags field "ifru_xflags" is a uint64_t which is not
 * a valid type for strict ANSI C compilation for ILP32. Applications that 
 * need access to extended flags field should be compiled in extended ANSI 
 * (-Ae) or LP64 mode.
 */
struct	ifreq {
#define	IFNAMSIZ	16
	char	ifr_name[IFNAMSIZ];		/* if name, e.g. "en0" */
	union {
		struct	sockaddr ifru_addr;
		struct	sockaddr ifru_dstaddr;
		struct	sockaddr ifru_broadaddr;
		short	ifru_flags;
#if defined(__STDC_EXT__) || defined(__LP64__)
		uint64_t ifru_xflags;		/* Extended flags */
#endif
		int	ifru_metric;
		caddr_t	ifru_data;
	} ifr_ifru;
#define	ifr_addr	ifr_ifru.ifru_addr	/* address */
#define	ifr_dstaddr	ifr_ifru.ifru_dstaddr	/* other end of p-to-p link */
#define	ifr_broadaddr	ifr_ifru.ifru_broadaddr	/* broadcast address */
#if defined(__STDC_EXT__) || defined(__LP64__)
#define ifr_xflags      ifr_ifru.ifru_xflags    /* extended flags */
#endif
#define	ifr_flags	ifr_ifru.ifru_flags	/* flags */
#define	ifr_metric	ifr_ifru.ifru_metric	/* metric */
#define ifr_index       ifr_ifru.ifru_metric    /* index */
#define	ifr_data	ifr_ifru.ifru_data	/* for use by interface */
};

#define SIOCGIFXFLAGS    _IOWR('i',38, struct ifreq)    /* get extended flags */
#define SIOCSIFXFLAGS    _IOW('i', 37, struct ifreq)    /* set extended flags */

/* Logical interface request */
struct lifreq {
   int  lifr_flags;
   char lifr_target[IFNAMSIZ];
   char lifr_source[IFNAMSIZ];
   };

/*
 * Structure used in SIOCGIFCONF request.
 * Used to retrieve interface configuration
 * for machine (useful for programs which
 * must know all networks accessible).
 */
struct	ifconf {
	int	ifc_len;		/* size of associated buffer */
	union {
		caddr_t	ifcu_buf;
		struct	ifreq *ifcu_req;
	} ifc_ifcu;
#define	ifc_buf	ifc_ifcu.ifcu_buf	/* buffer address */
#define	ifc_req	ifc_ifcu.ifcu_req	/* array of structures returned */
};

/*HP*/
/* These next two ioctls are being added here because we want to release
 * this code in a patch so it is too late to get this added into ioctl.h.
 * Hopefully we will get these into ioctl.h in the next release.
 */
#ifndef SIOCGIFTUNPARMS
#define SIOCGIFTUNPARMS  _IOWR('i', 35, struct iftun_parms) /*mip tunnl parms*/
#endif
#ifndef SIOCSIFTUNPARMS
#define SIOCSIFTUNPARMS  _IOW('i', 36, struct iftun_parms) /*mip tunnl parms*/
#endif

/* Tunnel parameters structure used by SIOCG/SIFTUNPARMS */
struct iftun_parms {
        char    iftp_name[IFNAMSIZ];            /* v4tu0:nnn interface name */
        struct sockaddr_storage iftp_tunnel;    /* Outer header src or dst */
        struct sockaddr_storage iftp_node;      /* Node which tunnel is for */
        uint32_t        iftp_ttl;               /* 0 for kernel default */
        uint32_t        iftp_type;              /* Type of tunnel, see below */
        uint32_t        iftp_flags;             /* Misc. flags, see below */
        uint32_t        iftp_pmtu;              /* Soft path MTU, "get" only */
        uint32_t        iftp_status;            /* Soft status, "get" only */
   };

/* iftp_flags values for iftun_parms */
#define IFTP_F_NOTIFY_FORWARDS  0x01    /* "upcall" on pkt forwarding */
#define IFTP_F_ANY_INBOUND_OK   0x02    /* Any inbound tunnel OK */
#define IFTP_F_SOURCE_FORWARDING 0x04   /* Tunnel pkts via src addr */
#define IFTP_F_XUNICAST_ENCAP   0x08    /* broad/multicast get encap. */
#define IFTP_F_TUNNEL_ENCAP     0x10    /* Tunnel if rcv'ed encap. */
#define IFTP_F_INBOUND_ONLY     0x20    /* To authorize inbound pkts */
#define IFTP_F_FLOAT_OUTERSRC   0x40    /* Null src addr, let routing choose. */
#define IFTP_F_ANY_NODE_OK      0x80    /* Allow any inner packet src/dst. */

/* iftp_type values for iftun_parms */
#define IFTP_TUNNEL_IPINIP      1       /* IP-in-IP tunneling */
#define IFTP_TUNNEL_MINENCAP    2       /* Minimal encapsulation tunnel */
#define IFTP_TUNNEL_GRE         4       /* GRE */

/* iftp_status values for iftun_parms */
#define IFTP_F_STATUS_UNREACH   0x01    /* recent "unreachable" status */
#define IFTP_F_STATUS_HOPLIMIT  0x02    /* recent "hop limit" status */

/*
 * Notification structure for tunnel "upcall" events to application.  This is
 * used in conjunction with the IP_TUNNEL_NOTIFIES socket option along with
 * the above tunnel forwarding flag.
 */
struct iftun_notify {
        struct sockaddr_storage iftn_src;       /* src addr of forwarded pkt */
        struct sockaddr_storage iftn_dst;       /* dst addr of forwarded pkt */
        uint32_t        iftn_event;             /* Event reason */
        char    iftn_name[IFNAMSIZ];            /* v4tu0:nnn interface name */
};
/* iftn_event events - currently only one */
#define IFTN_EVENT_FORWARD      1

#ifdef _KERNEL
#ifdef __LP64__ 
/*
 * Variant structure for 32 bit over 64 bit functionality.
 */
struct ifconf32 {
        int       ifc_len;              /* size of associated buffer */
	ptr32_t   ifc_buf32;
};

#define SIOCGIFCONF32   _IOWR('i',20, struct ifconf32)  /* get ifnet list */

#endif /* __LP64__ */
#endif /* _KERNEL */

/* 
 * structure for SIOC[A,D]IFADDR ioctls 
 */
struct ifaliasreq {
        char    ifra_name[IFNAMSIZ];            /* if name, e.g. "en0" */
        struct  sockaddr ifra_addr;
        struct  sockaddr ifra_broadaddr;
        struct  sockaddr ifra_mask;
};

#include <net/if_arp.h>
#include <net/if6.h>

#define SIOCSIFSWITCH   _IOW('i',29, struct ifreq)  /* HP internal switch IF */

#define SIOCGNEXTIFIDX  _IOWR('i',70, int) /* JAGad66016: HP internal request */
                                           /* to get next IFINDEX number */

#endif  /* _NET_IF_INCLUDED */
